"use strict";

const ee = require("./eventemitter");
let layers = [];

/**
 * Register a layer
 *   el: An object with an "isOpen" method
 *   lockEvent: Name of event that will initiate scroll lock (e.g. when layer has finished opening)
 *   unlockEvent: Name of event that will release scroll lock (e.g. when layer has finished closing)
 */
function registerLayer(el, lockEvent, unlockEvent) {
  let lockEv = lockEvent || null;
  let unlockEv = unlockEvent || null;
  layers.push({
    "element": el,
    "lockEvent": lockEv,
    "unlockEvent": unlockEv
  });
  if (lockEv) {
    ee.addListener(lockEv, lockScrolling);
  }
  if (unlockEv) {
    ee.addListener(unlockEv, unlockScrolling);
  }
}

function unregisterLayer(el) {
  let filtered = layers.filter((layer, index, array) => {
    if (layer.element === el) {
      if (layer.lockEvent) {
        ee.removeListener(layer.lockEvent, lockScrolling);
      }
      if (layer.unlockEvent) {
        ee.removeListener(layer.unlockEvent, unlockScrolling);
      }
      return false;
    }
    return true;
  });
}

function lockScrolling() {
  console.log("lockScrolling");
  window.document.body.classList.add("u-no-scroll");
}

/**
 * When a layer has finished closing, check whether it's safe to allow
 * the body to scroll again (all layers must be closed)
 */
function unlockScrolling() {
  let allLayersClosed = true;
  layers.map((currentValue, index, array) => {
    console.log(currentValue.element);
    console.log("isOpen: " + currentValue.element.isOpen());
    if (currentValue.element.isOpen()) {
      allLayersClosed = false;
    }
  });
  if (allLayersClosed) {
    console.log("unlockScrolling");
    window.document.body.classList.remove("u-no-scroll");
  }
  console.log("--------------");
}

module.exports = {
  registerLayer,
  unregisterLayer,
  lockScrolling,
  unlockScrolling
};
